dnl Lurupa, a library for verified linear programming.
dnl Copyright (C) 2006 by Christian Keil
dnl
dnl This file is part of Lurupa.
dnl
dnl Lurupa is free software; you can redistribute it and/or
dnl modify it under the terms of the GNU Lesser General Public
dnl License as published by the Free Software Foundation; either
dnl version 2.1 of the License, or (at your option) any later version.
dnl
dnl This library is distributed in the hope that it will be useful,
dnl but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
dnl Lesser General Public License for more details.
dnl
dnl You should have received a copy of the GNU Lesser General Public
dnl License along with this library; if not, write to the Free Software
dnl Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
dnl USA

# CK_CHECK_CXX_LIB(LIBRARY, PROLOGUE, BODY,
#                  [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND],
#                  [OTHER-LIBRARIES])
# ----------------------------------------------------------
#
# Checks if the program generated from the PROLOGUE and BODY arguments can be
# linked with the library LIBRARY. LIBRARY is the basename of the library; the
# macro tries to link the test program using -lLIBRARY.
# If this succeeds the shell commandos ACTION-IF-FOUND are executed otherwise
# the commandos ACTION-IF-NOT-FOUND. The default for ACTION-IF-FOUND is to
# prepend -lLIBRARY to the LIBS variable and define HAVE_LIBLIBRARY (with all
# capital letters).
# OTHER-LIBRARIES may be used if linking against LIBRARY would cause undefined
# references that are not resolved by the libraries present in LIBS at the time
# of invocation. This should be limited to the case where this can not be
# circumvented by rearranging the order of library checks.
AC_DEFUN([CK_CHECK_CXX_LIB],
    [AC_LANG_PUSH(C++)
    m4_ifval([$4], , [AH_CHECK_LIB([$1])])dnl
    AS_VAR_PUSHDEF([ck_Lib], [ck_cv_lib_$1])
    AC_CACHE_CHECK([if test program can be linked with -l$1], ck_Lib,
      [ck_check_cxx_lib_save_LIBS=$LIBS
      LIBS="-l$1 $6 $LIBS"
      AC_LINK_IFELSE([AC_LANG_PROGRAM([$2], [$3])],
        [AS_VAR_SET(ck_Lib, yes)],
        [AS_VAR_SET(ck_Lib, no)])
      LIBS=$ck_check_cxx_lib_save_LIBS])
    AS_IF([test AS_VAR_GET(ck_Lib) = yes],
      [m4_default([$4], [AC_DEFINE_UNQUOTED(AS_TR_CPP(HAVE_LIB$1))
        LIBS="-l$1 $LIBS"
        ])],
      [$5])dnl
    AS_VAR_POPDEF([ck_Lib])
    AC_LANG_POP(C++)dnl
    ])# CK_CHECK_CXX_LIB

# CK_SOLVER_WITH(SOLVER-ID, SOLVER-NAME, LIBRARIES, PROLOGUE, BODY)
# -----------------------------------------------------------------
# Handles the request to build a solver module.
# SOLVER-ID determines the names of the --with switches:
# --with-[SOLVER-ID],
# --with-[SOLVER-ID]-inc-path, and
# --with-[SOLVER-ID]-lib-path,
# supplied parameters are strings of `-l', `-I' and `-L' switches, respectively.
# SOLVER-NAME is the full name of the solver used in the description of the
# switches.
# If the --with-[SOLVER-ID] is not specified while one of the others is, a
# warning message is printed, that the solver module won't be built.
# The macro tries to compile a test program consisting of PROLOGUE and a main
# function with body BODY, prepending the user supplied values, with a default
# of LIBRARIES for the --with-[SOLVER-ID] switch, to the LIBS, CPPFLAGS, and
# LDFLAGS variables. If this succeeds the original variables are restored and
# the solver specific values are AC_SUBST'ed into LIB_[SOLVER-ID],
# INC_PATH_[SOLVER-ID], and, LIB_PATH_[SOLVER-ID].
AC_DEFUN([CK_SOLVER_WITH],
    [AC_ARG_WITH([$1],
      [AC_HELP_STRING([--with-$1=LIBRARIES],
        [compile $2 solver module using LIBRARIES, if
        LIBRARIES is not present default to `$3'; space
        seperated list of `-l' switches])],
      [AS_IF([test x"$AS_TR_SH(with_$1)" = x"yes"],
        [AS_TR_SH(with_$1)=$3])])
    AC_ARG_WITH([$1-inc-path],
      [AC_HELP_STRING([--with-$1-inc-path=PATH],
        [include path for $2 header files; space seperated
        list of `-I' switches])],
      [], [AS_TR_SH([with_$1-inc-path])=])
    AC_ARG_WITH([$1-lib-path],
      [AC_HELP_STRING([--with-$1-lib-path=PATH],
        [library path for $2 libraries; space seperated list
        of `-L' switches])],
      [], [AS_TR_SH([with_$1-lib-path])=])
# We have the switches, now check the arguments' consistency
    AS_IF([test -z "$AS_TR_SH([with_$1])" -a -n "$AS_TR_SH([with_$1-inc-path])"],
      [AC_MSG_WARN([[--with-$1 not specified, will not build module!!!]])])
  AS_IF([test -z "$AS_TR_SH([with_$1])" -a -n "$AS_TR_SH([with_$1-lib-path])"],
      [AC_MSG_WARN([[--with-$1 not specified, will not build module!!!]])])
  dnl Now off to the macro logic
# Enable the solver module and check for the solver if requested
  AM_CONDITIONAL(AS_TR_CPP([WITH_$1]), 
      test -n "$AS_TR_SH([with_$1])" -a ! x"$AS_TR_SH([with_$1])" = x"no")
  AS_IF([test -n "$AS_TR_SH([with_$1])" -a ! x"$AS_TR_SH([with_$1])" = x"no"],
# Try to link a program using the supplied switches
      [AS_VAR_PUSHDEF([ck_solver], [ck_cv_solver_$1])
      AC_CACHE_CHECK([if test program can be linked to $2], ck_solver,
        [ck_solver_with_save_CPPFLAGS=$CPPFLAGS
        ck_solver_with_save_LIBS=$LIBS
        ck_solver_with_save_LDFLAGS=$LDFLAGS
        CPPFLAGS="$AS_TR_SH([with_$1-inc-path]) $CPPFLAGS"
        LIBS="$AS_TR_SH([with_$1]) $LIBS"
        LDFLAGS="$AS_TR_SH([with_$1-lib-path]) $LDFLAGS"
        AC_LINK_IFELSE([AC_LANG_PROGRAM([$4], [$5])],
          [AS_VAR_SET(ck_solver, yes)],
          [AS_VAR_SET(ck_solver, no)])
        CPPFLAGS=$ck_solver_with_save_CPPFLAGS
        LIBS=$ck_solver_with_save_LIBS
        LDFLAGS=$ck_solver_with_save_LDFLAGS])
      AS_IF([test AS_VAR_GET(ck_solver) = yes],
        [AC_SUBST(AS_TR_CPP([LIB_$1]), [$AS_TR_SH([with_$1])])
        AC_SUBST(AS_TR_CPP([INC_PATH_$1]), [$AS_TR_SH([with_$1-inc-path])])
        AC_SUBST(AS_TR_CPP([LIB_PATH_$1]), [$AS_TR_SH([with_$1-lib-path])])],
        [AC_MSG_FAILURE([cannot link to $2])])
  AS_VAR_POPDEF([ck_solver])dnl
  ])])
