/*
 * Lurupa, a library for verified linear programming.
 * Copyright (C) 2006 by Christian Keil
 *
 * This file is part of Lurupa.
 *
 * Lurupa is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 * USA
 */

/**
 * @file  cLurupa.cpp
 * @brief Definition of C wrapper for %Lurupa
 *
 * This is work in progress.
 *
 * @author Christian Keil
 *
 * $Id: cLurupa.cpp 533 2008-06-12 20:08:12Z keil $
 */
extern "C" {
#include <lurupa/cLurupa.h>
}
#include <lurupa/Lurupa.h>
#include <lurupa/Lp.h>

/** @brief Lurupa instance used by C wrapper */
static Lurupa l;

/**
 * Initialize the library wrapper.
 */
void
clu_init()
{
  /*
  l = new Lurupa;
  */
}

/**
 * Cleanup the library wrapper.
 */
void
clu_cleanup()
{
  /*
  delete l;
  */
}

/**
 * Get the core version string.
 *
 * @sa
 * Lurupa#get_core_version
 */
const char *
clu_get_core_version()
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.get_core_version();
}

/**
 * Get the algorithm parameter alpha.
 *
 * @sa
 * Lurupa#get_alpha
 */
double
clu_get_alpha()
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.get_alpha();
}

/**
 * Set the algorithm parameter alpha.
 *
 * @sa
 * Lurupa#set_alpha
 */
void
clu_set_alpha(double alpha)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  l.set_alpha(alpha);
}

/**
 * Get the algorithm parameter eta.
 *
 * @sa
 * Lurupa#get_eta
 */
double
clu_get_eta()
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.get_eta();
}

/**
 * Set the algorithm parameter eta.
 *
 * @sa
 * Lurupa#set_eta
 */
void
clu_set_eta(double eta)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  l.set_eta(eta);
}

/**
 * Get the algorithm flag inflate.
 *
 * @sa
 * Lurupa#is_inflate
 */
BOOL
clu_is_inflate()
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.is_inflate();
}

/**
 * Set the algorithm flag inflation.
 *
 * @sa
 * Lurupa#set_inflate
 */
void
clu_set_inflate(BOOL inflate)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  l.set_inflate(inflate);
}

/**
 * Set the solver module.
 *
 * @sa
 * Lurupa#set_module
 */
BOOL
clu_set_module(char *module_path)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.set_module(module_path);
}

/**
 * Set the solver module.
 *
 * @sa
 * Lurupa#set_interface
 */
void
clu_set_interface(Solver_module_interface interface)
{
#if 0
   if (!l) {
    clu_init();
  }
#endif
  l.set_interface(interface);
}

/**
 * Get the solver module version.
 *
 * @sa
 * Lurupa#get_module_version
 */
const char *
clu_get_module_version()
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.get_module_version();
}

/**
 * Print the options supported by the module.
 *
 * @sa
 * Lurupa#print_module_options
 */
void
clu_print_module_options()
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  l.print_module_options();
}

/**
 * Get solver accuracy.
 *
 * @sa
 * Lurupa#get_solver_eps
 */
double
clu_get_solver_eps()
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.get_solver_eps();
}

/**
 * Compute the lower bound.
 *
 * @sa
 * Lurupa#lower_bound
 */
Bound_status 
clu_lower_bound(PLP lp, double *bound, int *iterations)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.lower_bound(lp, *bound, *iterations);
}

/**
 * Compute the upper bound.
 *
 * @sa
 * Lurupa#upper_bound
 */
Bound_status
clu_upper_bound(PLP lp, double *bound, int *iterations)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.upper_bound(lp, *bound, *iterations);
}

/**
 * Get the linear program's name.
 *
 * @sa
 * Lurupa#get_lp_name
 */
char *
clu_get_lp_name(PLP lp)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return lp->name;
}

/**
 * Return whether the linear program is maximized.
 *
 * @sa
 * Lurupa#is_lp_maximize
 */
BOOL
clu_is_lp_maximize(PLP lp)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return lp->maximize;
}

/**
 * Read a linear program.
 *
 * @sa
 * Lurupa#read_lp
 */
PLP
clu_read_lp(FILE *in, const double relative_interval_radius)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.read_lp(in, relative_interval_radius);
}

BOOL
clu_set_solution(PLP lp, void *p)
{
#if 0
   if (!l) {
    clu_init();
  }
#endif
  return l.set_solution(lp, p);
}

/**
 * Set a linear program.
 *
 * @sa
 * Lurupa#set_lp
 */
BOOL
clu_set_lp(PLP lp, const double relative_interval_radius)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.set_lp(lp, relative_interval_radius);
}

/**
 * Set module options.
 *
 * @sa
 * Lurupa#set_module_options
 */
BOOL
clu_set_module_options(PLP lp, int argc, char *argv[])
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.set_module_options(lp, argc, argv);
}

/**
 * Solve the linear program.
 *
 * @sa
 * Lurupa#solve_lp
 */
BOOL
clu_solve_lp(PLP lp, double *optimal_value)
{
#if 0
   if (!l) {
   clu_init();
  }
#endif
  return l.solve_lp(lp, *optimal_value);
}
