% LURUPA - Linear UnderestimateR Using Perturbed Approximations
%
% Lurupa computes verified results for Linear Programs (LP). This includes
% rigorous bounds on the optimal value as well as rigorous enclosures of near
% optimal, feasible points.
%
% The LP has to be specified as a structure. The names of the structure's fields
% are unimportant what matters is their ordering. The first field has to contain
% the direction of optimization (c). Next come the matrix of the inequality
% constraints (A) and the corresponding right hand sides (a). After that the
% matrix of equations (B) and their right hand sides (b). Finally the simple
% lower (xl) and upper bounds (xu). Using the denoted field names of structured
% variable lp, this is interpreted as the LP
%
%   min  lp.c' * x
%   s.t. lp.A * x <= a
%        lp.B * x <= b
%        lp.xl <= x <= lp.xu
%
% The usage depends on the first parameter specified when calling LURUPA.
%
% FL = LURUPA('lower_bound',lp) computes the verified lower bound FL on the
% optimal value of the LP lp.
%
% [FL,FLAG,YL,YU,ZL,ZU] = LURUPA('lower_bound',lp) additionally returns FLAG
% denoting the status of the bound computation in the following way:
%    0  Bound successfully computed
%    1  Bound successfully computed; problem is verified to be infeasible 
%    2  Bound successfully computed; problem is verified to be unbounded 
%    3  Bound not computed; the lp-solver claims infeasibility 
%    4  Bound not computed; the lp-solver claims unboundedness 
%    5  Bound not computed; a perturbed problem seems to be infeasible and
%         inflation is not set 
%    6  Bound not computed; no enclosure of the solution set of linear system
%         could be found 
%    7  Bound not computed; the lp-solver gave a warning while solving a
%         perturbed problem 
%    8  Bound not computed; the equation matrix has not full rank 
%    9  Bound not computed; the maximum number of iterations is exceeded 
%   10  Bound not computed; an unexpected error occured 
%   11  Bound not computed; the lp-solver timed out solving a perturbed problem 
% It further returns enclosures of near optimal lagrange parameters Y of the
% inequality constraints and Z of the equations. These satisfy YL <= Y <= YU and
% ZL <= Z <= ZU, where
% <= holds componentwise.
%
% [FL,FLAG,YL,YU,ZL,ZU,F,X] = LURUPA('lower_bound',lp) computes the previous
% bounds and returns the approximate optimal value and solution in F and X,
% respectively.
%
% FU = LURUPA('upper_bound',lp) computes the verified upper bound FU on the
% optimal value of the LP lp.
%
% [FU,FLAG,XL,XU] = LURUPA('upper_bound',lp) additionally returns the status of
% the bound computation in FLAG and an enclosure of near optimal feasible points
% X with XL <= X <= XU. These vector inequalities again hold componentwise.
%
% [FU,FLAG,XL,XU,F,X] = LURUPA('upper_bound',lp) computes the previous bounds
% and returns the approximate optimal value and solution in F and X,
% respectively.
%
% Copyright (C) 2008 by Christian Keil
%   $Revision$

% $Id$

function lurupa(x1, x2)
mexfunc = ['lurupa.' mexext];
disp(['Could not find ' mexfunc '!'])
disp('Make sure that:')
disp('  1) you have compiled the sources')
disp('      (execute the `makelurupamex'' script)')
disp(['  2) ' mexfunc ' resides in the current directory or on the MATLAB path'])
error([mexfunc ' not found'])



